#####################################################################################################################
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2003-2011.  All Rights Reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
######################################################################################################################
# 
# Updated by: Shirley Dango
# Updated on: April 21, 2003 
# Updated by Katie Carpenter, October & November 2007
# Updated by Stuart Poulin Nov 2007
#
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user			The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname			The schema repository connection to the database
#	dbset			The user database connection
#	pvob			The name of the project vob
#	baseline		The name of the baseline
#	build			The name of the build
#	buildstatus		The type of build 
#	projectid		The CQ id of the project this build is related to - for example ALM0000045
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
# thl3   RATLC00475598       10/19/2005      THL               SQUID Scripts for baseline and build create fail when 
#                                                              login is ldap enabled
# thl4   RATLC00475646       10/19/2005      THL               Need to check for \ on pvob parm 
# thl5   RATLC00475658       10/23/2005      THL               SQUID Script create_build_record allows multiple build 
#                                                              records to be created with the same build name 
#                                                              (Build field value)
#			     10/11/2007	     katherns	       Modify references of 'Build' to 'BTBuild'
#
#        RATLC00396109       10/26/2007      katherns          build cannot be created against an obsolete baseline
#
#        RATLC00396140       10/26/2007      katherns          build record does not need to contain activities
#
#        RATLC00396143       10/26/2007      katherns          change references of 'relCWActivities' to 'Activities'
#
#        RATLC01214353       10/26/2007      katherns          add Build_System_ID as a parameter (build)
#
#        RATLC01216069       11/5/2007       katherns          add ALMBuildType as a parameter
#
#        RATLC00396394       1/14/2008       katherns          Update scripts for M3 release.
#        RATLC               3/31/2008       spoulin           add -url option
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
my $InternalVersion = 0.6;
use strict;
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use CQPerlExt;
use Env;
use IO::File;
use LWP::Simple;
use POSIX;
use Time::Local;
my ($entity);
my $id;
my ($failure);
my ($baselineentity);
my $result;
my $progname = basename $0;
my $status ;
my $buildstatus ; #thl2
my $userLogin ; #thl2
my ($resultset,$resultsetstatus,$buildId);
my $recordOwner;

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $result = &main();

if ($result) {
    print "ʶ $id\n";
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

####
# Subroutine to display the usage
sub Usage {
	print "ʹ CQ API ¼ʵó
    ÷
       $progname -test
         -help
         -user <û>
         -pw <>
         -dbname <ݿ>
         -dbset <ݿ⼯>
         -pvob <pvob>
         -baseline <>
         -build <>
         -buildstatus <״̬>
         -buildtype <>
         -projectid <Ŀʶ>
	 -logfile <־ļ>
	 -url <Ҫ URL>
"		
}

##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}

sub main {
####
# Define the input parameters
my @opts = qw(
    user=s
    pw=s
    dbname=s
    dbset=s
    pvob=s
    baseline=s
    build=s
    buildstatus=s
    buildtype=s
    projectid=s
    owner=s
    logfile=s
    test
    help
    url=s
   
);


my %opts = (
    user => $ENV{ALMUSER},
    pw => $ENV{ALMPW},
    dbname => $ENV{ALMDBNAME},
    dbset => $ENV{ALMDBSET},
    pvob => $ENV{ALMPVOB},
    baseline => undef,
    build =>undef,
    buildstatus => undef,
    buildtype => undef,
    projectid => undef,
    owner => undef,
    logfile => undef,
    url=>undef,
);


my @required_opts = qw ( user pw dbname dbset pvob baseline buildstatus buildtype projectid build );

####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
 	$logger->log_print_message("ʾ÷");	
      	my $retval = &Usage;
      	return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error( __LINE__ . "޷־ļĿ¼$logdir");
            		$logger->log_print_error( __LINE__ . "űڲĿ¼¼ִС");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error( __LINE__ . "޷־ļ$opts{logfile}");
        	$logger->log_print_error( __LINE__ . "űڲĿ¼¼ִС");
   	 }
	 $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("¼ $log_time ʼ"); 
}	 

####
# Display the help usage
if ($opts{help}) {
	print "-user		          ClearQuest ûĵ¼ʶ\n";
	print "-pw		            ClearQuest û롣\n";
	print "-dbname		        ݿģʽ洢ӡ\n";
	print "-dbset		         ûݿӡ\n";
	print "-pvob		          Ŀ VOB ơ\n";
	print "-baseline	       ߵơ\n";
	print "-build		         ơ\n";
	print "-buildstatus	    ״̬\n";
        print "-buildtype       ͡\n";
	print "-projectid	      ùصĿıʶ\n";
	print "-logfile		       ڶ־ļ·Ŀѡ־ȱʡڿ̨ʾ\n";
	print "-url		           Ϊڴݹ URL Ŀѡ־\n";
	print "-test		          ΪļõĿѡ־\n";
	print "-help            ڰĿѡ־\n";
    return 1;
}

# Display test inputs to log file 03/26/03
if ($opts{test}){
	$logger->log_print_message("Ѷû룺");
	$logger->log_print_message("Ѷû¼Ϊ $opts{user}");
	$logger->log_print_message("Ѷݿ⼯Ϊ $opts{dbset}");
	$logger->log_print_message("ѶݿΪ $opts{dbname}");
	$logger->log_print_message("ѶĻΪ $opts{baseline}");
	$logger->log_print_message("ѶĿ VOB Ϊ $opts{pvob}");
	$logger->log_print_message("ѶĹ״̬Ϊ $opts{buildstatus}");
	$logger->log_print_message("ѶĹΪ $opts{buildtype}");
	$logger->log_print_message("ѶĿʶΪ $opts{projectid}");
	if ($opts{logfile}){ $logger->log_print_message("Ѷ־ļΪ $opts{logfile}");}
	return 1;
}

####
####
# End program if any of the required parameters are not defined
{my  $message = "";
	for my $req (@required_opts )  {
		$message .= "ȱٱѡ -$req \n" if ! defined $opts{$req};
	}
	if ($message ) {
		$logger->log_print_message("ȱٱѡʾ÷");
		$logger->log_print_message("$message");
		print STDERR "$message\n";
		my $retval = &Usage();
		return 0;
	}
}
;


my ($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});
###
# Start a CQ session
my $session = CQPerlExt::CQSession_Build();

# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
# Logon to the specified database
eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
$logger->log_print_error( __LINE__ . "¼ $opts{dbname} ʧܣ$@"); 
return 0;
}
$userLogin = $session->GetUserLoginName() ; #thl3
$logger->log_print_message("ѳɹ¼ $opts{dbname} $userLogin"); #thl3


# See if the build record already exists and return if it does            thl5
my $queryDef = $session->BuildQuery("BTBuild");                            #thl5   
$failure = $queryDef->BuildField("id");                                  #thl5
my $operator = $queryDef->BuildFilterOperator($CQPerlExt::CQ_BOOL_OP_AND); #thl5 
$operator->BuildFilter("OldID", $CQPerlExt::CQ_COMP_OP_EQ, ["$opts{build}"]); #thl5
eval {$resultset = $session->BuildResultSet($queryDef)};                  #thl5
if ($@) {                                                                 #thl5
   $logger->log_print_error(__LINE__ . ": ѯʧܣ$@");   #thl5
   return 0;                                                              #thl5
}                                                                         #thl5
else {                                                                    #thl5
   $resultset->Execute();                                                 #thl5
   $resultsetstatus = $resultset->MoveNext;                               #thl5
   if ($resultsetstatus == $CQPerlExt::CQ_SUCCESS) {                      #thl5
      $buildId = $resultset->GetColumnValue(1);
      $logger->log_print_error(__LINE__ . ": Ѵڡ$buildIdĹ¼ $opts{build}");#thl5
      return 0;                                                           #thl5
   }                                                                      #thl5
}                                                                         #thl5

# Find the Baseline record in the ClearQuest database
my $baselineentity = undef;
for (1) {
	eval qq{\$baselineentity = \$session->GetEntity("ALMBaseline", \$noslashvob.' '.\$opts{baseline})}; 
	if ($baselineentity) {
		$opts{pvob} = $noslashvob;
		last;
	}
	eval qq{\$baselineentity = \$session->GetEntity("ALMBaseline", \$slashvob.' '.\$opts{baseline})}; 
	if ($baselineentity) {
		$opts{pvob} = $slashvob;
		last;
	}
	$baselineentity = $session->GetEntity("ALMBaseline", $opts{pvob}.' '.$opts{baseline}); 
}

$logger->log_print_message("Ŀ VOB$opts{pvob}ҵ $opts{baseline} Ļʵ塣");

# Determine if the Baseline record was found in ClearQuest database
if ($baselineentity){

#Do not create build if baseline is obsolete
my $obsoletevalue = $baselineentity->GetFieldValue("Obsolete")->GetValue();
if($obsoletevalue eq 'Yes'){
$logger->log_print_message(" ߼¼״̬ΪObsoleteδ¼");
$logger->log_print_error(" ߼¼״̬ΪObsoleteδ¼");
return 0;
}

	$logger->log_print_message("ڴµĹ¼");
	
	## Build the Build record in ClearQuest database
	my $entity;
	eval {$entity = $session->BuildEntity("BTBuild")};
	if ($@) {
		$logger->log_print_error( __LINE__ . "޷򿪹¼"); 
		$logger->log_print_error( __LINE__ . "$@");
        return 0;
	}


	my $baselinename = $opts{pvob} . ' ' . $opts{baseline};
	
	## Set field values for pvob, baseline, buildstatus,project, and version
	
	$entity->SetFieldValue("ALMProject", $opts{projectid});
	$entity->SetFieldValue("Build_System_ID", $opts{build});	
	$entity->SetFieldValue("ALMBuildStatus",$opts{buildstatus});
        $entity->SetFieldValue("ALMBuildType", $opts{buildtype});
	$entity->SetFieldValue("ALMOwner", $userLogin); 
        $entity->SetFieldValue("ALMBaseline", $baselinename); 
	$entity->SetFieldValue("Build_System_URL", $opts{url});
			
	## New Build ID
	$id = $entity->GetFieldValue("id")->GetValue();
	$logger->log_print_message("ʶ=$id");
	$logger->log_print_message("֤µĹ¼");
	$status = "" ; #thl2
	eval {$status = $entity->Validate()};
	if (($@) || ($status)) {           #thl2
		$entity->Revert();
		$logger->log_print_error(__LINE__ . "޷֤ ClearQuest ݿеĹ¼");
		$logger->log_print_error(__LINE__ . "  eval=$@");
                $logger->log_print_error(__LINE__ . "ص״̬Ϊ $status"); #thl2
		$logger->log_print_error(__LINE__ . "ڹ¼˳򣩣");
		$logger->log_print_error("\tALMProject", $opts{projectid});
		$logger->log_print_error("\tBuild_System_ID", $opts{build});	
		$logger->log_print_error("\tALMBuildStatus",$opts{buildstatus});
		$logger->log_print_error("\tALMBuildType", $opts{buildtype});
		$logger->log_print_error("\tALMOwner", $userLogin); 
		$logger->log_print_error("\tALMBaseline", $baselinename); 
		return 0;
	}
        $status = "" ; #thl2
	$logger->log_print_message("ʵµĹ¼");
	eval {$status = $entity->Commit()}; #thl2
	if (($@) || ($status)) {            #thl2
		$entity->Revert();
		$logger->log_print_error( __LINE__ . "޷ ClearQuest ݿʵ¼");
		$logger->log_print_error( __LINE__ . "$@");
                $logger->log_print_error(__LINE__ . "ص״̬Ϊ $status"); #thl2
		CQPerlExt::CQSession_Unbuild($session);
		return 0;
	}
        # Write to log file
	$logger->log_print_message("ѴµĹ¼");
	$entity = undef;
	CQPerlExt::CQSession_Unbuild($session);
	} 
else {
	# Write to log file 
	$logger->log_print_message("δҵ߼¼봴һµĻ߼¼");
	$logger->log_print_message("δκι¼");	
	CQPerlExt::CQSession_Unbuild($session);
	return 0;
} # end of creating Build and updating Baseline record in CQ
# Write to log file

$logger->log_print_success(" create_build_record ű");
$logger->log_print_message("create_build_record.pl script ű");
CQPerlExt::CQSession_Unbuild($session);

return 1;
}
#### SUBROUTINES FOR PROGRAM
